/*
Copyright 2017 Vector Creations Ltd
Copyright 2017 New Vector Ltd

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import React from 'react';
import PropTypes from 'prop-types';
import sdk from '../../../index';
import MatrixClientPeg from '../../../MatrixClientPeg';
import GeminiScrollbar from 'react-gemini-scrollbar';
import Resend from '../../../Resend';
import { _t } from '../../../languageHandler';
import SettingsStore from "../../../settings/SettingsStore";

function markAllDevicesKnown(devices) {
    Object.keys(devices).forEach((userId) => {
        Object.keys(devices[userId]).map((deviceId) => {
            MatrixClientPeg.get().setDeviceKnown(userId, deviceId, true);
        });
    });
}

function DeviceListEntry(props) {
    const {userId, device} = props;

    const DeviceVerifyButtons = sdk.getComponent('elements.DeviceVerifyButtons');

    return (
        <li>
            <DeviceVerifyButtons device={device} userId={userId} />
            { device.deviceId }
            <br />
            { device.getDisplayName() }
        </li>
    );
}

DeviceListEntry.propTypes = {
    userId: PropTypes.string.isRequired,

    // deviceinfo
    device: PropTypes.object.isRequired,
};


function UserUnknownDeviceList(props) {
    const {userId, userDevices} = props;

    const deviceListEntries = Object.keys(userDevices).map((deviceId) =>
       <DeviceListEntry key={deviceId} userId={userId}
           device={userDevices[deviceId]} />,
    );

    return (
        <ul className="mx_UnknownDeviceDialog_deviceList">
            { deviceListEntries }
        </ul>
    );
}

UserUnknownDeviceList.propTypes = {
    userId: PropTypes.string.isRequired,

    // map from deviceid -> deviceinfo
    userDevices: PropTypes.object.isRequired,
};


function UnknownDeviceList(props) {
    const {devices} = props;

    const userListEntries = Object.keys(devices).map((userId) =>
        <li key={userId}>
            <p>{ userId }:</p>
            <UserUnknownDeviceList userId={userId} userDevices={devices[userId]} />
        </li>,
    );

    return <ul>{ userListEntries }</ul>;
}

UnknownDeviceList.propTypes = {
    // map from userid -> deviceid -> deviceinfo
    devices: PropTypes.object.isRequired,
};


export default React.createClass({
    displayName: 'UnknownDeviceDialog',

    propTypes: {
        room: PropTypes.object.isRequired,

        // map from userid -> deviceid -> deviceinfo or null if devices are not yet loaded
        devices: PropTypes.object,

        onFinished: PropTypes.func.isRequired,

        // Label for the button that marks all devices known and tries the send again
        sendAnywayLabel: PropTypes.string.isRequired,

        // Label for the button that to send the event if you've verified all devices
        sendLabel: PropTypes.string.isRequired,

        // function to retry the request once all devices are verified / known
        onSend: PropTypes.func.isRequired,
    },

    componentWillMount: function() {
        MatrixClientPeg.get().on("deviceVerificationChanged", this._onDeviceVerificationChanged);
    },

    componentWillUnmount: function() {
        if (MatrixClientPeg.get()) {
            MatrixClientPeg.get().removeListener("deviceVerificationChanged", this._onDeviceVerificationChanged);
        }
    },

    _onDeviceVerificationChanged: function(userId, deviceId, deviceInfo) {
        if (this.props.devices[userId] && this.props.devices[userId][deviceId]) {
            // XXX: Mutating props :/
            this.props.devices[userId][deviceId] = deviceInfo;
            this.forceUpdate();
        }
    },

    _onDismissClicked: function() {
        this.props.onFinished();
    },

    _onSendAnywayClicked: function() {
        markAllDevicesKnown(this.props.devices);

        this.props.onFinished();
        this.props.onSend();
    },

    _onSendClicked: function() {
        this.props.onFinished();
        this.props.onSend();
    },

    render: function() {
        if (this.props.devices === null) {
            const Spinner = sdk.getComponent("elements.Spinner");
            return <Spinner />;
        }

        let warning;
        if (SettingsStore.getValue("blacklistUnverifiedDevices", this.props.room.roomId)) {
            warning = (
                <h4>
                    { _t("You are currently blacklisting unverified devices; to send " +
                    "messages to these devices you must verify them.") }
                </h4>
            );
        } else {
            warning = (
                <div>
                    <p>
                        { _t("We recommend you go through the verification process " +
                            "for each device to confirm they belong to their legitimate owner, " +
                            "but you can resend the message without verifying if you prefer.") }
                    </p>
                </div>
            );
        }

        let haveUnknownDevices = false;
        Object.keys(this.props.devices).forEach((userId) => {
            Object.keys(this.props.devices[userId]).map((deviceId) => {
                const device = this.props.devices[userId][deviceId];
                if (device.isUnverified() && !device.isKnown()) {
                    haveUnknownDevices = true;
                }
            });
        });
        let sendButton;
        if (haveUnknownDevices) {
            sendButton = <button onClick={this._onSendAnywayClicked}>
                { this.props.sendAnywayLabel }
            </button>;
        } else {
            sendButton = <button onClick={this._onSendClicked}>
                { this.props.sendLabel }
            </button>;
        }

        const BaseDialog = sdk.getComponent('views.dialogs.BaseDialog');
        return (
            <BaseDialog className='mx_UnknownDeviceDialog'
                onFinished={this.props.onFinished}
                title={_t('Room contains unknown devices')}
            >
                <GeminiScrollbar autoshow={false} className="mx_Dialog_content">
                    <h4>
                        { _t('"%(RoomName)s" contains devices that you haven\'t seen before.', {RoomName: this.props.room.name}) }
                    </h4>
                    { warning }
                    { _t("Unknown devices") }:

                    <UnknownDeviceList devices={this.props.devices} />
                </GeminiScrollbar>
                <div className="mx_Dialog_buttons">
                    {sendButton}
                    <button className="mx_Dialog_primary" autoFocus={true}
                        onClick={this._onDismissClicked}
                    >
                        {_t("Dismiss")}
                    </button>
                </div>
            </BaseDialog>
        );
        // XXX: do we want to give the user the option to enable blacklistUnverifiedDevices for this room (or globally) at this point?
        // It feels like confused users will likely turn it on and then disappear in a cloud of UISIs...
    },
});
